/*
 * The MIT License
 *
 * Copyright 2013 Masahiko, SAWAI <masahiko.sawai@gmail.com>.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package com.example.hello.android.notification_hello;

import android.app.Activity;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.Toast;

public class MainActivity extends Activity
	implements OnClickListener
{

	private static final int NOTIFICATION_ID_HELLO_SIMPLE = 1;
	private NotificationManager notificationManager;

	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);
		notificationManager = (NotificationManager) getSystemService(Activity.NOTIFICATION_SERVICE);

		Button notifyButton = (Button) findViewById(R.id.notify_button);
		notifyButton.setOnClickListener(this);

		Button notifyWithVibrate = (Button) findViewById(R.id.notify_with_vibrate_button);
		notifyWithVibrate.setOnClickListener(this);

		Button cancelButton = (Button) findViewById(R.id.cancel_button);
		cancelButton.setOnClickListener(this);

		Button cancelAllButton = (Button) findViewById(R.id.cancel_all_button);
		cancelAllButton.setOnClickListener(this);
	}

	@Override
	protected void onPause()
	{
		notificationManager.cancelAll();
		super.onPause();
	}

	public void onClick(View v)
	{
		switch (v.getId())
		{
			case R.id.notify_button:
				notifySimple();
				break;
			case R.id.notify_with_vibrate_button:
				notifyWithVibrate();
				break;
			case R.id.cancel_button:
				cancelSimpleNotification();
				break;
			case R.id.cancel_all_button:
				cancelAllNotification();
				break;
			default:
				throw new AssertionError();
		}
	}

	private void notifySimple()
	{
		// 通知をタップしたときに送信されるインテントを作成
		Intent intent = new Intent(this, HelloActivity.class);
		PendingIntent pendingIntent = PendingIntent.getActivity(this, 0, intent, 0);

		// NotificationBuilderを作成
		Notification.Builder builder = new Notification.Builder(
			getApplicationContext());
		builder.setContentIntent(pendingIntent);

		// 通知したときにステータスバーに表示されるテキストを設定
		builder.setTicker("Hello, simple notification");
		// 通知したときにステータスバーに表示されるアイコン
		builder.setSmallIcon(android.R.drawable.ic_dialog_info);

		// Notificationを開いたときに表示されるタイトル
		builder.setContentTitle("Simple Event Title");
		// Notificationを開いたときに表示されるサブタイトル
		builder.setContentText("Simple Event Text");
		// 通知するタイミング
		builder.setWhen(System.currentTimeMillis());
		// 通知時の音
		builder.setDefaults(Notification.DEFAULT_SOUND);
		// タップするとキャンセル(消える)
		builder.setAutoCancel(true);
	}

	private void notifyWithVibrate()
	{
		// 通知をタップしたときに送信されるインテントを作成
		Intent intent = new Intent(this, HelloActivity.class);
		PendingIntent pendingIntent = PendingIntent.getActivity(this, 0, intent, 0);

		// NotificationBuilderを作成
		Notification.Builder builder = new Notification.Builder(
			getApplicationContext());
		builder.setContentIntent(pendingIntent);

		// 通知したときにステータスバーに表示されるテキストを設定
		builder.setTicker("Nofity with vibration");
		// 通知したときにステータスバーに表示されるアイコン
		builder.setSmallIcon(android.R.drawable.ic_dialog_info);

		// Notificationを開いたときに表示されるタイトル
		builder.setContentTitle("Vibration Event Title");
		// Notificationを開いたときに表示されるサブタイトル
		builder.setContentText("Vibration Event Text");
		// 通知するタイミング
		builder.setWhen(System.currentTimeMillis());
		// 通知時の音・バイブ・ライト
		builder.setDefaults(Notification.DEFAULT_SOUND
			| Notification.DEFAULT_VIBRATE
			| Notification.DEFAULT_LIGHTS);
		// タップするとキャンセル(消える)
		builder.setAutoCancel(true);

		// Notificationを作成して通知
		notificationManager.notify(NOTIFICATION_ID_HELLO_SIMPLE, builder.build());
	}

	private void cancelSimpleNotification()
	{
		Toast.makeText(this, R.string.cancel, Toast.LENGTH_SHORT).show();
		notificationManager.cancel(NOTIFICATION_ID_HELLO_SIMPLE);
	}

	private void cancelAllNotification()
	{
		Toast.makeText(this, R.string.cancel_all, Toast.LENGTH_SHORT).show();
		notificationManager.cancelAll();
	}
}
